<?xml version="1.0" encoding="UTF-8"?>
<ncx xmlns="http://www.daisy.org/z3986/2005/ncx/" version="2005-1">
  <head>
    <meta name="dtb:uid" content="{{ uid }}"/>
    <meta name="dtb:depth" content="1"/>
    <meta name="dtb:totalPageCount" content="0"/>
    <meta name="dtb:maxPageNumber" content="0"/>
  </head>
  <docTitle><text>{{ epub.title }}</text></docTitle>
  <docAuthor><text>{{ epub.creator }}</text></docAuthor>
  <navMap>
    {%- for (assign, chapter) in chapters %}
    <navPoint id="{{ assign.id }}" playOrder="{{ assign.play_order }}">
      <navLabel><text>{{ chapter.title }}</text></navLabel>
      <content src="{{ assign.link }}"/>
    </navPoint>
    {%- endfor %}
  </navMap>
</ncx>
