<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xmlns:epub="http://www.idpf.org/2007/ops" xml:lang="en">
  <head>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8"/>
    <title>Table of Contents</title>
  </head>
  <body>
    <h2>Table of Contents</h2>
    <hr/>
    <div id="chapters">
      {%- for (assign, chapter) in chapters %}
      <p><a href= "{{ assign.link }}">{{ chapter.title | e }}</a></p>
      {%- endfor %}
    </div>
  </body>
</html>
