<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xmlns:epub="http://www.idpf.org/2007/ops" xml:lang="en">
  <head>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8"/>
    <link type="text/css" rel="stylesheet" href="styles/styles.css"/>
    <link type="text/css" rel="stylesheet" href="styles/coverpage.css" />
		{%-for css in css_paths%}
    <link type="text/css" rel="stylesheet" href="styles/{{css | basename }}"/>
    {%-endfor%}
    <title>Cover Image</title>
	</head>
	<body>
		<div>
			<img src="{{ cover }}" alt="Cover image" style="width: 100%;height: 100%"/>
		</div>
	</body>
</html>
